<?php
/**
 *
 * @package phpBB Extension - glacier_phpbb_compare
 * @copyright (c) 2023 - GlacierParkChat
 * @license GNU General Public License, version 2 (GPL-2.0)
 *
 */

namespace glacierparkchat\glacier_phpbb_compare\acp;

include __DIR__ . '/../vendor/autoload.php';


/**
* @package acp
*/

class config_module
{
	/** @var string */
	public $u_action;
	/** @var string */
	public $tpl_name;
	/** @var string */
	public $page_title;

	function main($id, $mode)
	{
		global $db, $user, $template, $request, $phpbb_log, $phpbb_container, $cache, $config, $language;

		
		$initialIgnorelist = explode(";", $config['gpc_phpbb_compare_top_level_ignore_list']);
		foreach ($initialIgnorelist as $key => $val)
		{
		    $ignoreList[$key] = explode(':', $val);
		    $template->assign_block_vars('phpbb_compare_ignore_list', [
		        'FOLDER'	  => $ignoreList[$key][0],
		        'VAL'         => $ignoreList[$key][1],
		    ]);
		    
		    
		}
		
		
		$template->assign_vars([
		    'PHPBB_COMPARE_SOURCE_PATH'	    => $config['gpc_phpbb_compare_source_path'],
		    'PHPBB_COMPARE_COMPARE_PATH'	=> $config['gpc_phpbb_compare_compare_path'],
		    'PHPBB_COMPARE_HISTORY_MAX'	    => $config['gpc_phpbb_compare_history_max'],
		]);
		
		
		
		
		$this->tpl_name = 'config_module';
		$language->add_lang('phpbb_compare', 'glacierparkchat/glacier_phpbb_compare');
		$this->page_title = $language->lang('ACP_PHPBB_COMPARE_TITLE');
		$diffhistory_table = $phpbb_container->getParameter('tables.diffhistory_table');
        $current_id = 0;
		$formkey = 'phpbb_compare/phpbb_compare';
		
		add_form_key($formkey);

		
		
		if ($request->is_set_post('submitSourceDefaultReset'))
		{
		    if (!check_form_key($formkey))
		    {
		        trigger_error('FORM_INVALID');
		    }
		    
		    $rootDir = realpath($_SERVER["DOCUMENT_ROOT"]);
		    $rootScriptPath = $user->page['root_script_path'];
		    
		    $sourcePath = $rootDir . $rootScriptPath;
		    $config->set('gpc_phpbb_compare_source_path', $sourcePath);
		    
		    $user_id = $user->data['user_id'];
		    $user_ip = $user->ip;
		    
		    $phpbb_log->add('admin', $user_id, $user_ip, 'ACP_PHPBB_COMPARE_SAVED');
		    trigger_error($language->lang('ACP_PHPBB_COMPARE_SAVED') . adm_back_link($this->u_action));
		    
		}
		elseif ($request->is_set_post('submitUpdate'))
		{
		    if (!check_form_key($formkey))
		    {
		        trigger_error('FORM_INVALID');
		    }
		    
		    $src = $request->variable('phpbb_compare_source', '');
		    $compare = $request->variable('phpbb_compare_compare', '');
		    
		    // add trailing slash if missing
		    if(substr($src, -1) != "/") {
		        $src .= "/";
		    }
		    // add trailing slash if missing
		    if(substr($compare, -1) != "/") {
		        $compare .= "/";
		    }
		    
		    
		    $config->set('gpc_phpbb_compare_source_path', $src);
		    $config->set('gpc_phpbb_compare_compare_path', $compare);
		    
		    
		    
		    //$cache->destroy('sql', $menuinclude_table);
		    
		    $user_id = $user->data['user_id'];
		    $user_ip = $user->ip;
		    
		    $phpbb_log->add('admin', $user_id, $user_ip, 'ACP_PHPBB_COMPARE_SAVED');
		    trigger_error($language->lang('ACP_PHPBB_COMPARE_SAVED') . adm_back_link($this->u_action));
		}
		elseif ($request->is_set_post('submitMax'))
		{
		    $config->set('gpc_phpbb_compare_history_max', $request->variable('gpc_phpbb_compare_history_max', 0));
		    
		    
		    $cache->destroy('sql', $diffhistory_table);
		    
		    $user_id = $user->data['user_id'];
		    $user_ip = $user->ip;
		    
		    $phpbb_log->add('admin', $user_id, $user_ip, 'ACP_PHPBB_COMPARE_SAVED');
		    trigger_error($language->lang('ACP_PHPBB_COMPARE_SAVED') . adm_back_link($this->u_action));
		}
		elseif ($request->is_set_post('submitIgnoreList'))
		{
		    foreach ($ignoreList as $key => $val)
		    {
		        $v = $request->variable('ignore_'.$val[0], 0);
		        
		        $val[1] = $v;
		        $ignoreList[$key] = $val;
		        
		        $ignoreListImplode[] = implode(":",[$val[0],$val[1]]);
		    }

		    $config->set('gpc_phpbb_compare_top_level_ignore_list', implode(";",$ignoreListImplode));
		    
		    $user_id = $user->data['user_id'];
		    $user_ip = $user->ip;
		    
		    $phpbb_log->add('admin', $user_id, $user_ip, 'ACP_PHPBB_COMPARE_SAVED');
		    trigger_error($language->lang('ACP_PHPBB_COMPARE_SAVED') . adm_back_link($this->u_action));
		}

		
		
	}
	
	

	

}
