<?php
/**
 *
 * @package phpBB Extension - glacier_phpbb_compare
 * @copyright (c) 2023 - GlacierParkChat
 * @license GNU General Public License, version 2 (GPL-2.0)
 *
 */

namespace glacierparkchat\glacier_phpbb_compare\acp;

include __DIR__ . '/../vendor/autoload.php';

use Jfcherng\Diff\DiffHelper;
use Jfcherng\Diff\Renderer\RendererConstant;

/**
* @package acp
*/

class process_module
{
	/** @var string */
	public $u_action;
	/** @var string */
	public $tpl_name;
	/** @var string */
	public $page_title;

	public $totals = [
	    'inserted'     => 0,
	    'deleted'      => 0,
	    'mixed'        => 0,
	    'nochange'     => 0,
	    'unknown'      => 0,
	    'new_file'     => 0,
	    'file_deleted' => 0,
	];
	
	public $diffTypes = [
	     //'NO_CHANGE'  => ,
	    'UNKNOWN'      => -1,
	    'DELETED'      => 0,
	    'INSERTED'     => 1,
	    'MIXED'        => 2,
	    'NEW_FILE'     => 3,
	    'FILE_DELETED' => 4,
	];
	
	function main($id, $mode)
	{
		global $db, $user, $template, $request, $phpbb_log, $phpbb_container, $cache, $config, $language;

		//global $symfony_request, $phpbb_filesystem;
		
		$sourcePath = $config['gpc_phpbb_compare_source_path'];
		$comparePath = $config['gpc_phpbb_compare_compare_path'];
		
		$template->assign_vars([
		    'PHPBB_COMPARE_SOURCE_PATH'	    => $sourcePath,
		    'TOTALS_INSERTED'               => $config['gpc_phpbb_compare_totals_inserted'],
            'TOTALS_DELETED'                => $config['gpc_phpbb_compare_totals_deleted'],
            'TOTALS_MIXED'                  => $config['gpc_phpbb_compare_totals_mixed'],
            'TOTALS_NOCHANGE'               => $config['gpc_phpbb_compare_totals_nochange'],
            'TOTALS_UNKNOWN'                => $config['gpc_phpbb_compare_totals_unknown'],
		    'TOTALS_TOTAL'                  => $config['gpc_phpbb_compare_totals_total'],
		    'TOTALS_NEW_FILE'               => $config['gpc_phpbb_compare_totals_new_file'],
		    'TOTALS_FILE_DELETED'           => $config['gpc_phpbb_compare_totals_file_deleted'],
		    'PHPBB_COMPARE_RESULT_DATE'     => $config['gpc_phpbb_compare_result_date'],
		]);
		
		$skipList = array();
		
		//$rootDir = realpath($_SERVER["DOCUMENT_ROOT"]);
		//$rootScriptPath = $user->page['root_script_path'];
		
		$initialIgnorelist = explode(";", $config['gpc_phpbb_compare_top_level_ignore_list']);
		foreach ($initialIgnorelist as $key => $val)
		{
		    $ignoreList[$key] = explode(':', $val);
		    //$template->assign_block_vars('phpbb_compare_ignore_list', [
		    //    'FOLDER'	  => $ignoreList[$key][0],
		    //    'VAL'         => $ignoreList[$key][1],
		    //]);
		    
		    if($ignoreList[$key][1] == 1)
		    {
		        $skipList[] = $config['gpc_phpbb_compare_source_path'].$ignoreList[$key][0].'/';
		    }
		}
		
		
		
		$this->tpl_name = 'process_module';
		$language->add_lang('phpbb_compare', 'glacierparkchat/glacier_phpbb_compare');
		$this->page_title = $language->lang('ACP_PHPBB_COMPARE_TITLE');
		$diffhistory_table = $phpbb_container->getParameter('tables.diffhistory_table');
        //$current_id = 0;
		$formkey = 'phpbb_compare/phpbb_compare';
		
		add_form_key($formkey);

		$sql = 'SELECT  id, filename, top_level, diff, diff_type, compare_id, compare_timestamp, file_type, file_size, last_mod
		FROM '. $diffhistory_table .
		' order by top_level desc, filename asc ';
		$result = $db->sql_query($sql);
		
		while ($row = $db->sql_fetchrow($result))
		{
		    if (!empty($row['id']))
		    {
		        //$current_id = $row['compare_id'];
		        $template->assign_block_vars('phpbb_compare', [
		            'ID'					=> $row['id'],
		            'FILENAME'				=> $row['filename'],
		            'TOP_LEVEL'             => $row['top_level'],
		            'DIFF'                  => $row['diff'],
		            'DIFF_TYPE'             => $row['diff_type'],
		            'COMPARE_ID'            => $row['compare_id'],
		            'COMPARE_TIMESTAMP'	    => $row['compare_timestamp'],
		            'FILE_TYPE'	            => $row['file_type'],
		            //'FILE_SIZE'	            => number_format($row['file_size']),
		            'FILE_SIZE'	            => $this->human_filesize($row['file_size']),
		            'LAST_MOD'	            => date('m/d/Y h:i:s A', $row['last_mod']),
		        ]);
		    }
		};

		
		$db->sql_freeresult($result);
		
		
	
		if ($request->is_set_post('submitProcess'))
		{
		    $this->totals = [
		        'inserted'     => 0,
		        'deleted'      => 0,
		        'mixed'        => 0,
		        'nochange'     => 0,
		        'unknown'      => 0,
		        'new_file'     => 0,
		        'file_deleted' => 0,
		    ];
		    
		    
		    // Clear out Diff History table....
		    $sql = 'DELETE FROM ' . $diffhistory_table ;
		    $db->sql_query($sql);
		    
		    //Process Files
		    $this->getFileList($sourcePath, $skipList, TRUE, 1);
		    
		    //Process Files the other way only to check for delete files
		    $this->processInReverseForDeletes($comparePath, $skipList, TRUE, 1);
		    
		    $config->set('gpc_phpbb_compare_totals_inserted', number_format($this->totals['inserted']));
		    $config->set('gpc_phpbb_compare_totals_deleted', number_format($this->totals['deleted']));
		    $config->set('gpc_phpbb_compare_totals_mixed', number_format($this->totals['mixed']));
		    $config->set('gpc_phpbb_compare_totals_nochange', number_format($this->totals['nochange']));
		    $config->set('gpc_phpbb_compare_totals_unknown', number_format($this->totals['unknown']));
		    $config->set('gpc_phpbb_compare_totals_total', number_format($this->totals['inserted']+$this->totals['deleted']+$this->totals['mixed']+$this->totals['nochange']+$this->totals['unknown']+$this->totals['new_file']+$this->totals['file_deleted']));
		    $config->set('gpc_phpbb_compare_totals_new_file', number_format($this->totals['new_file']));
		    $config->set('gpc_phpbb_compare_totals_file_deleted', number_format($this->totals['file_deleted']));
		    $config->set('gpc_phpbb_compare_result_date' , date('m/d/Y h:i:s A',strtotime("now")));
		    
		    
		    $user_id = $user->data['user_id'];
		    $user_ip = $user->ip;
		    
		    $phpbb_log->add('admin', $user_id, $user_ip, 'ACP_PHPBB_COMPARE_PROCESSED');
		    trigger_error($language->lang('ACP_PHPBB_COMPARE_PROCESSED') . adm_back_link($this->u_action));
		    
		    $this->totals = [
		        'inserted'     => 0,
		        'deleted'      => 0,
		        'mixed'        => 0,
		        'nochange'     => 0,
		        'unknown'      => 0,
		        'new_file'     => 0,
		        'file_deleted' => 0,
		    ];
		}		    		
	
		
	}
	
	function human_filesize($bytes, $decimals = 2) {
	    $sz = 'BKMGTP';
	    $factor = floor((strlen($bytes) - 1) / 3);
	    return sprintf("%.{$decimals}f", $bytes / pow(1024, $factor)) . @$sz[$factor];
	}
	
	
	function getFileList($dir, $skipList, $recurse = FALSE, $topLevel = 0)
	{
	    global $config, $phpbb_container, $db;
	    
	    $diffhistory_table = $phpbb_container->getParameter('tables.diffhistory_table');
	    
	    $diffOptions = [
	        // show how many neighbor lines
	        // Differ::CONTEXT_ALL can be used to show the whole file
	        'context' => 1,
	        // ignore case difference
	        'ignoreCase' => false,
	        // ignore line ending difference
	        'ignoreLineEnding' => false,
	        // ignore whitespace difference
	        'ignoreWhitespace' => false,
	        // if the input sequence is too long, it will just gives up (especially for char-level diff)
	        'lengthLimit' => 2000,
	    ];
	    
	    // options for renderer class  $rendererOptions = ['detailLevel' => 'line'];
	    $rendererOptions = [
	        // how detailed the rendered HTML is? (none, line, word, char)
	        'detailLevel' => 'line',
	        // renderer language: eng, cht, chs, jpn, ...
	        // or an array which has the same keys with a language file
	        // check the "Custom Language" section in the readme for more advanced usage
	        'language' => 'eng',
	        // show line numbers in HTML renderers
	        'lineNumbers' => true,
	        // show a separator between different diff hunks in HTML renderers
	        'separateBlock' => true,
	        // show the (table) header
	        'showHeader' => true,
	        // convert spaces/tabs into HTML codes like `<span class="ch sp"> </span>`
	        // and the frontend is responsible for rendering them with CSS.
	        // when using this, "spacesToNbsp" should be false and "tabSize" is not respected.
	        'spaceToHtmlTag' => false,
	        // the frontend HTML could use CSS "white-space: pre;" to visualize consecutive whitespaces
	        // but if you want to visualize them in the backend with "&nbsp;", you can set this to true
	        'spacesToNbsp' => false,
	        // HTML renderer tab width (negative = do not convert into spaces)
	        'tabSize' => 4,
	        // this option is currently only for the Combined renderer.
	        // it determines whether a replace-type block should be merged or not
	        // depending on the content changed ratio, which values between 0 and 1.
	        'mergeThreshold' => 0.8,
	        // this option is currently only for the Unified and the Context renderers.
	        // RendererConstant::CLI_COLOR_AUTO = colorize the output if possible (default)
	        // RendererConstant::CLI_COLOR_ENABLE = force to colorize the output
	        // RendererConstant::CLI_COLOR_DISABLE = force not to colorize the output
	        'cliColorization' => RendererConstant::CLI_COLOR_AUTO,
	        // this option is currently only for the Json renderer.
	        // internally, ops (tags) are all int type but this is not good for human reading.
	        // set this to "true" to convert them into string form before outputting.
	        'outputTagAsString' => false,
	        // this option is currently only for the Json renderer.
	        // it controls how the output JSON is formatted.
	        // see available options on https://www.php.net/manual/en/function.json-encode.php
	        'jsonEncodeFlags' => \JSON_UNESCAPED_SLASHES | \JSON_UNESCAPED_UNICODE,
	        // this option is currently effective when the "detailLevel" is "word"
	        // characters listed in this array can be used to make diff segments into a whole
	        // for example, making "<del>good</del>-<del>looking</del>" into "<del>good-looking</del>"
	        // this should bring better readability but set this to empty array if you do not want it
	        'wordGlues' => [' ', '-'],
	        // change this value to a string as the returned diff if the two input strings are identical
	        'resultForIdenticals' => null,
	        // extra HTML classes added to the DOM of the diff container
	        'wrapperClasses' => ['diff-wrapper'],
	    ];
	    
	    $retval = [];
	    $sql_ary = [];
	    
	    // add trailing slash if missing
	    if(substr($dir, -1) != "/") {
	        $dir .= "/";
	    }
	    
	    
	    // open pointer to directory and read list of files
	    $d = @dir($dir) or die("getFileList: Failed opening directory [$dir] for reading");
	    
	    while(FALSE !== ($entry = $d->read())) {
	        
	        // skip hidden files
	        if($entry[0] == ".") {
	            continue;
	        }
	        
	        if(is_dir($dir.$entry)) {
	            /*
	            $retval[] = [
	                'name' => $dir.$entry.'/',
	                'type' => filetype($dir.$entry),
	                'size' => 0,
	                'lastmod' => filemtime($dir.$entry)
	            ];
	            */
	            if($recurse && is_readable($dir.$entry.'/') && !in_array($dir.$entry.'/', $skipList)) {
	                //$retval = array_merge($retval, $this->getFileList($dir.$entry.'/', $skipList, $recurse, 0));
	                $this->getFileList($dir.$entry.'/', $skipList, $recurse, 0);
	            }
	            
	        } elseif(is_readable($dir.$entry)) {
	            /*
	            $retval[] = [
	                'name' => $dir.$entry,
	                'type' => mime_content_type($dir.$entry),
	                'size' => filesize($dir.$entry),
	                'lastmod' => filemtime($dir.$entry)
	            ];
	            */
	            
	            $sourcePath = $config['gpc_phpbb_compare_source_path'];
	            $comparePath = $config['gpc_phpbb_compare_compare_path'];
	            $compareDir = $comparePath.substr($dir, strlen($sourcePath));
	            
	            $newFile = $dir.$entry;  // Source file
	            $oldFile = $compareDir.$entry;  // Compare file
	            
	            
	            if(is_readable($oldFile))
	            {
	                // generate a combined diff
	                $sideBySideResult = DiffHelper::calculateFiles(
	                    $oldFile,
	                    $newFile,
	                    'SideBySide',
	                    $diffOptions,
	                    $rendererOptions,
	                    );
	                
	                
	                
	                if(strlen($sideBySideResult) > 0)
	                {
	                    // Found something....
	                    $stats = DiffHelper::getStatistics();
	                    
	                    if($stats['inserted'] > 0 && $stats['deleted'] > 0)
	                    {
	                        $diffType = $this->diffTypes['MIXED'];
	                        $this->totals['mixed']++;
	                    }
	                    elseif($stats['inserted'] > 0 )
	                    {
	                        $diffType = $this->diffTypes['INSERTED'];
	                        $this->totals['inserted']++;
	                    }
	                    elseif($stats['deleted'] > 0 )
	                    {
	                        $diffType = $this->diffTypes['DELETED'];
	                        $this->totals['deleted']++;
	                    }
	                    else
	                    {
	                        $diffType = $this->diffTypes['UNKNOWN'];
	                        $this->totals['unknown']++;
	                    }
	                    
	                    
	                    $sql_ary[] = [
	                        'filename' 		     => substr($newFile, strlen($sourcePath)),
	                        'top_level'          => $topLevel,
	                        'diff'				 => $sideBySideResult,
	                        'diff_type'          => $diffType,
	                        'compare_timestamp'  => strtotime("now"),
	                        'file_type'          => mime_content_type($dir.$entry),
	                        'file_size'          => filesize($dir.$entry),
	                        'last_mod'           => filemtime($dir.$entry),
	                    ];
	                    //$db->sql_multi_insert($diffhistory_table,  $sql_ary);
	                    
	                }
	                else
	                {
	                    //files match - no change?
	                    $this->totals['nochange']++;
	                    
	                }
	            }
	            else
	            {
	                // This is a new file....
	                // It doesn't exist on the other side..
	                // Need to deal with that...
	                $newFilePart = substr($newFile, strlen($sourcePath));
	                $sql_ary[] = [
	                    'filename' 		     => $newFilePart,
	                    'top_level'          => $topLevel,
	                    'diff'				 => $this->newFileAdded($newFilePart),
	                    'diff_type'          => $this->diffTypes['NEW_FILE'],
	                    'compare_timestamp'  => strtotime("now"),
	                    'file_type'          => mime_content_type($dir.$entry),
	                    'file_size'          => filesize($dir.$entry),
	                    'last_mod'           => filemtime($dir.$entry),
	                ];
	                $this->totals['new_file']++;
	                //$db->sql_multi_insert($diffhistory_table,  $sql_ary);
	            }
	        }
	        
	    } // for each file
	    
	    $db->sql_multi_insert($diffhistory_table,  $sql_ary);
	    $d->close();
	    
	    return $retval;
	}
	
	function processInReverseForDeletes($dir, $skipList, $recurse = FALSE, $topLevel = 0)
	{
	    global $config, $phpbb_container, $db;
	    
	    $diffhistory_table = $phpbb_container->getParameter('tables.diffhistory_table');
	    
	    
	    $retval = [];
	    $sql_ary = [];
	    
	    // add trailing slash if missing
	    if(substr($dir, -1) != "/") {
	        $dir .= "/";
	    }
	    
	    
	    // open pointer to directory and read list of files
	    $d = @dir($dir) or die("getFileList: Failed opening directory [$dir] for reading");
	    
	    while(FALSE !== ($entry = $d->read())) {
	        
	        // skip hidden files
	        if($entry[0] == ".") {
	            continue;
	        }
	        
	        if(is_dir($dir.$entry)) {
	            /*
	            $retval[] = [
	                'name' => $dir.$entry.'/',
	                'type' => filetype($dir.$entry),
	                'size' => 0,
	                'lastmod' => filemtime($dir.$entry)
	            ];
	            */
	            if($recurse && is_readable($dir.$entry.'/') && !in_array($dir.$entry.'/', $skipList)) {
	                //$retval = array_merge($retval, $this->processInReverseForDeletes($dir.$entry.'/', $skipList, $recurse, 0));
	                $this->processInReverseForDeletes($dir.$entry.'/', $skipList, $recurse, 0);
	            }
	            
	        } elseif(is_readable($dir.$entry)) {
	            /*
	            $retval[] = [
	                'name' => $dir.$entry,
	                'type' => mime_content_type($dir.$entry),
	                'size' => filesize($dir.$entry),
	                'lastmod' => filemtime($dir.$entry)
	            ];
	            */
	            // Swapping paths since running in reverse
	            $sourcePath = $config['gpc_phpbb_compare_compare_path'];
	            $comparePath = $config['gpc_phpbb_compare_source_path'];
	            $compareDir = $comparePath.substr($dir, strlen($sourcePath));
	            
	            $newFile = $dir.$entry;  // Source file
	            $oldFile = $compareDir.$entry;  // Compare file
	            
	            
	            if(is_readable($oldFile))
	            {
	                // Do nothing.... This has already been processed the other way
	            }
	            else
	            {
	                // This is a deleted file....
	                // Mark that it's been deleted...
	                $newFilePart = substr($newFile, strlen($sourcePath));
	                $sql_ary[] = [
	                    'filename' 		     => $newFilePart,
	                    'top_level'          => $topLevel,
	                    'diff'				 => $this->fileDeleted($newFilePart),
	                    'diff_type'          => $this->diffTypes['FILE_DELETED'],
	                    'compare_timestamp'  => strtotime("now"),
	                    'file_type'          => mime_content_type($dir.$entry),
	                    'file_size'          => filesize($dir.$entry),
	                    'last_mod'           => filemtime($dir.$entry),
	                ];
	                $this->totals['file_deleted']++;
	                //$db->sql_multi_insert($diffhistory_table,  $sql_ary);
	            }
	        }
	        
	    } // for each file
	    
	    $db->sql_multi_insert($diffhistory_table,  $sql_ary);
	    $d->close();
	    
	    return $retval;
	}
	
	function newFileAdded($newFilePart)
	{
	    $str = '
	    <table class="diff-wrapper diff diff-html diff-side-by-side">
	    <thead>
	    <tr>
	    <th colspan="2">Old</th>
	    <th colspan="2">New</th>
	    </tr>
	    </thead>
	    <tbody class="change change-eq">
	    <tr>
	    <th class="n-old"> </th>
	    <td class="old">    </td>
	    <th class="n-new"> </th>
	    <td class="new"> </td>
	    </tr>
	    </tbody>
	    <tbody class="change change-rep">
	    <tr>
	    <th class="n-old"> </th>
	    <td class="old"> <del>File Did Not Exist Before: '.$newFilePart.'</del>    </td>
	    <th class="n-new">+++</th>
	    <td class="new"> <ins>Brand New File Added: '.$newFilePart.'</ins>   </td>
	    </tr>
	    </tbody>
	    <tbody class="change change-eq">
	    <tr>
	    <th class="n-old"> </th>
	    <td class="old"></td>
	    <th class="n-new"> </th>
	    <td class="new"></td>
	    </tr>
	    </tbody>
	    </table>
        ';
	    
	    return $str;
	}
	
	function fileDeleted($newFilePart)
	{
	    $str = '
	    <table class="diff-wrapper diff diff-html diff-side-by-side">
	    <thead>
	    <tr>
	    <th colspan="2">Old</th>
	    <th colspan="2">New</th>
	    </tr>
	    </thead>
	    <tbody class="change change-eq">
	    <tr>
	    <th class="n-old"> </th>
	    <td class="old">    </td>
	    <th class="n-new"> </th>
	    <td class="new"> </td>
	    </tr>
	    </tbody>
	    <tbody class="change change-rep">
	    <tr>
	    <th class="n-old"> </th>
	    <td class="new"> <ins>File Existed Before: '.$newFilePart.'</ins>    </td>
	    <th class="n-new">---</th>
	    <td class="old"> <del>File Has Been Deleted: '.$newFilePart.'</del>   </td>
	    </tr>
	    </tbody>
	    <tbody class="change change-eq">
	    <tr>
	    <th class="n-old"> </th>
	    <td class="old"></td>
	    <th class="n-new"> </th>
	    <td class="new"></td>
	    </tr>
	    </tbody>
	    </table>
        ';
	    
	    return $str;
	}
	

}
