<?php
/**
*
* @package phpBB Extension - Glacier_phpBB_compare
* @copyright (c) 2023 Glacier Park Chat
* @license http://opensource.org/licenses/gpl-2.0.php GNU General Public License v2
*
*/

namespace glacierparkchat\glacier_phpbb_compare\migrations\v100x;

use phpbb\db\migration\container_aware_migration;

/**
 * Migration stage 1: Initial schema
 */
class m1_initial_file extends container_aware_migration
{
    /** @var string */
    public $diffhistory_table = 'gpc_diff_history';
    
	/**
	 * Assign migration file dependencies for this migration
	 *
	 * @return array Array of migration files
	 * @static
	 * @access public
	 */
	static public function depends_on()
	{
		return array('\phpbb\db\migration\data\v330\v330');
	}
	
	
	

	public function update_schema()
	{
	    return array(
	        'add_tables'	=> [
	            // ADD NEW TABLE STRUCTURE
	            $this->table_prefix . $this->diffhistory_table	=> [
	                'COLUMNS' => [
	                    'id'						=> ['UINT:11', null, 'auto_increment'],
	                    'filename'					=> ['VCHAR:255', null],
	                    'top_level'				    => ['BOOL', null],
	                    'diff'			     	    => ['MTEXT', null],
	                    'diff_type'                 => ['TINT:3', null],
	                    'compare_id'                => ['UINT', null],
	                    'compare_timestamp'         => ['TIMESTAMP', null],
	                    'file_type'                 => ['VCHAR:255', null],
	                    'file_size'                 => ['UINT:11', null],
	                    'last_mod'                  => ['TIMESTAMP', null],
	                ],
	                'PRIMARY_KEY'	=> 'id',
	            ],
	        ],
	    );
	}
	
	public function update_data()
	{
	    global $user, $request;
	    
	    $rootDir = realpath($request->server('DOCUMENT_ROOT'));
	   // $rootDir = realpath($_SERVER["DOCUMENT_ROOT"]);
	    $rootScriptPath = $user->page['root_script_path'];
	    
	    $sourcePath = $rootDir . $rootScriptPath;
	    
	    $ignoreList = array();
	    $scan = scandir($sourcePath);
	    foreach($scan as $file) 
	    {
	        if (is_dir($sourcePath.$file) && $file[0] != ".") 
	        {
	            $ignoreList[] = implode(":",[$file,0]);
	        }
	    }
	    
	    // ADD CONFIG
	    return [
	        ['config.add', ['gpc_phpbb_compare_history_max', '20']],
	        ['config.add', ['gpc_phpbb_compare_source_path', $sourcePath]],
	        ['config.add', ['gpc_phpbb_compare_top_level_ignore_list', implode(";",$ignoreList)]],
	        
	        // Add ACP modules
	        ['module.add', ['acp', 'ACP_CAT_DOT_MODS', 'ACP_PHPBB_COMPARE_TITLE']],
	        ['module.add', ['acp', 'ACP_PHPBB_COMPARE_TITLE', [
	            'module_basename'	=> '\glacierparkchat\glacier_phpbb_compare\acp\config_module',
	            'module_langname'	=> 'ACP_PHPBB_COMPARE',
	            'module_mode'		=> 'overview',
	            'module_auth'		=> 'ext_glacierparkchat/glacier_phpbb_compare && acl_a_board',
	        ]]],
	        ['module.add', ['acp', 'ACP_PHPBB_COMPARE_TITLE', [
	            'module_basename'	=> '\glacierparkchat\glacier_phpbb_compare\acp\process_module',
	            'module_langname'	=> 'ACP_PHPBB_COMPARE_PROCESS',
	            'module_mode'		=> 'overview',
	            'module_auth'		=> 'ext_glacierparkchat/glacier_phpbb_compare && acl_a_board',
	        ]]],
	        ['custom', [[$this, 'add_one_diff_history_row_to_start']]],
	    ];
	}
	
	/**
	 * 
	 * @return array Array of table schema
	 * @access public
	 */
	public function revert_schema()
	{
	    
	    return [
	        'drop_tables'	=> [
	            $this->table_prefix . $this->diffhistory_table,
	        ],
	    ];
	}
	
	public function add_one_diff_history_row_to_start()
	{
	    $sql = "INSERT INTO ".$this->table_prefix . $this->diffhistory_table." VALUES (1,null,null,null,null,null,null,null,null,null)";
	    
	    $this->sql_query($sql);
	}
}
